#ifndef DVM_DEV_H_INCLUDED
#define DVM_DEV_H_INCLUDED
#include "DVM.h"

typedef struct DVM_Array_tag DVM_Array;

typedef enum {
    DVM_SUCCESS = 1,
    DVM_ERROR
} DVM_ErrorStatus;

typedef DVM_Value DVM_NativeFunctionProc(DVM_VirtualMachine *dvm,
                                         int arg_count, DVM_Value *args);
/* execute.c */
void DVM_add_native_function(DVM_VirtualMachine *dvm,
                             char *package_name, char *func_name,
                             DVM_NativeFunctionProc *proc, int arg_count,
                             DVM_Boolean is_method,
                             DVM_Boolean return_pointer);
/* nativeif.c */
DVM_ErrorStatus
DVM_array_get_int(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index,
                  int *value, DVM_ObjectRef *exception_p);
DVM_ErrorStatus
DVM_array_get_double(DVM_VirtualMachine *dvm, DVM_ObjectRef array,
                     int index, double *value,
                     DVM_ObjectRef *exception_p);
DVM_ErrorStatus
DVM_array_get_object(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index,
                     DVM_ObjectRef *value, DVM_ObjectRef *exception_p);
DVM_ErrorStatus
DVM_array_set_int(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index,
                  int value, DVM_ObjectRef *exception_p);
DVM_ErrorStatus
DVM_array_set_double(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index,
                     double value, DVM_ObjectRef *exception_p);
DVM_ErrorStatus
DVM_array_set_object(DVM_VirtualMachine *dvm, DVM_ObjectRef array, int index,
                     DVM_ObjectRef value, DVM_ObjectRef *exception_p);
int DVM_array_size(DVM_VirtualMachine *dvm, DVM_Object *array);
void
DVM_array_resize(DVM_VirtualMachine *dvm, DVM_Object *array, int new_size);
void DVM_array_insert(DVM_VirtualMachine *dvm, DVM_Object *array, int pos,
                      DVM_Value value);
void DVM_array_remove(DVM_VirtualMachine *dvm, DVM_Object *array, int pos);
int DVM_string_length(DVM_VirtualMachine *dvm, DVM_Object *string);
DVM_Value DVM_string_substr(DVM_VirtualMachine *dvm, DVM_Object *str,
                            int pos, int len);
int DVM_get_field_index(DVM_VirtualMachine *dvm, DVM_ObjectRef obj,
                        char *field_name);

/* heap.c */
void DVM_check_gc(DVM_VirtualMachine *dvm);
DVM_ObjectRef DVM_create_array_int(DVM_VirtualMachine *dvm, int size);
DVM_ObjectRef DVM_create_array_double(DVM_VirtualMachine *dvm, int size);
DVM_ObjectRef DVM_create_array_object(DVM_VirtualMachine *dvm, int size);

#endif /* DVM_DEV_H_INCLUDED */
